/*  READS  Download to be used with R-31J, R535J, RMB-S Boards   */
/*  Copyright (C) 1994, Rigel Corporation, Gainesville, Florida  */

/*  RIGEL confidential -- for internal use only                  */
/*  skeletal code -- not for customer use                        */
/*  Compiled with Borland C++                                    */

#include <bios.h>
#include <conio.h>
#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define BufferSize    0x4000           /* 16k bytes.           */
#define BufferSize_1  0x3fff           /* BufferSize - 1       */
#define CR            13               /* carridge return      */
#define LF            10               /* line feed            */

/*--- global goodies --------------------------------------- */
unsigned char Com_ID=0;            // 0 for COM1, 1 for COM2
unsigned  int BaudRate = 9600;
unsigned char DataBits = 8;
unsigned char StopBits = 1;
unsigned char Parity   = 0;

unsigned int  com_params = 0xE3; // (111 00 0 11)
/*------------------------------------------------------------------*\
| $00e3 for default 9600 baud, 8 data bits, 1 stop bit, no parity.  |
| com_params=bbbppsll where                                         |
|	  bbb: baud rate 110,150,300,600,1200,2400,4800,9600        |
|          pp: parity 0:no, 1: odd, 3:even                          |
|           s: stop bit: 0 (1 stop bit) or 1 (2 stop bits)          |
|          ll: word length: 2 (7-bit) or 3 (8-bit)                  |
\*------------------------------------------------------------------*/

unsigned int  tailptr, headptr;
int           overflow;
unsigned char buffer[BufferSize];

unsigned int  com_dataport,
	      com_interruptenableport,
	      com_interruptidport,
	      com_linecontrolport,
	      com_modemcontrolport,
	      com_linestatusport,
	      com_modemstatusport;

int           iASCII=0;
/* ---- interrupt function vectors ---------------------------------------- */
void interrupt far (*oldhandler)(...);
/* ------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------- */
/* :::::::::::::: interrupt service routine :::::::::::::::::::::::::::::::: */
/* ------------------------------------------------------------------------- */
void interrupt far inthandler(...){ /* interrupt0bh handling routine */

 while(!(inportb(com_interruptidport) & 1))
  {
   if((inportb(com_interruptidport) & 0x06)==4)
    {
     if(tailptr<BufferSize_1) tailptr++; else tailptr=0;
     if(tailptr!=headptr) buffer[tailptr] = inportb(com_dataport);
       else
	{
	 if(tailptr>0) tailptr--; else tailptr=BufferSize_1;
	 overflow=1;
	 inportb(com_dataport);
	 }
     }
   }
 outportb(0x20, 0x20);
}
/* ------------------------------------------------------------------------ */
/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
/* ------------------------------------------------------------------------ */
void send(unsigned char b){
/* waits for transfer hold register empty */
 while(!(inportb(com_linestatusport) & 0x20)); /* until thre */
 outportb(com_dataport, b);
}
/* ------------------------------------------------------------------------ */
unsigned char receive(void){

 if(headptr != BufferSize) headptr++; else headptr = 0;
 return(buffer[headptr]);
}
/* ------------------------------------------------------------------------ */
int CharReady(void){return(headptr!=tailptr);}
/* ------------------------------------------------------------------------ */
int iCheckBoard(int iTimeOutPeriod){
          int iBoardFound;
unsigned char uc, ucPrompt;

 if(iASCII)ucPrompt='>';  // for BASIC
  else ucPrompt='*';      // for READS

 while(CharReady()) receive();

 send(CR);

 delay(iTimeOutPeriod);
 if(!CharReady())
  {
   printf("Board NOT Responding... Press RESET on Board\n");
   printf("RLoad Terminates...\n");
   iBoardFound=0;
   }
  else
    {
     uc=receive();
     printf("%c",uc);
     if(uc!=ucPrompt)
      while(CharReady())
       {
        uc=receive();
        printf("%c",uc);
        if(uc==ucPrompt)
         {
          iBoardFound=1;
          break;
          }
        }
       else iBoardFound=0;
     }

 delay(10);
 return(iBoardFound);
}
/* ------------------------------------------------------------------------ */
void com_setup(void){
struct REGPACK reg;

 switch(Com_ID)
  {
   case 1 : Com_ID = 0;
	    com_dataport            = 0x3f8;
	    com_interruptenableport = 0x3f9;
	    com_interruptidport     = 0x3fa;
	    com_linecontrolport     = 0x3fb;
	    com_modemcontrolport    = 0x3fc;
	    com_linestatusport      = 0x3fd;
	    com_modemstatusport     = 0x3fe;
	    break;

  case 2 :  Com_ID = 1;
	    com_dataport            = 0x2f8;
	    com_interruptenableport = 0x2f9;
	    com_interruptidport     = 0x2fa;
	    com_linecontrolport     = 0x2fb;
	    com_modemcontrolport    = 0x2fc;
	    com_linestatusport      = 0x2fd;
	    com_modemstatusport     = 0x2fe;
	    break;
  default:  printf("Error : Serial Port ID be 1 or 2\n");
	    exit(1);
    }

 oldhandler = getvect(0x0b);	       /* save the old interrupt vector */
 setvect(0x0b, inthandler);            /* install interrupt 0ch vector     */
 outportb(0x21, inportb(0x21) & 0xf7); /* 8259 mask - enable int 0bh/ir 3  */

 reg.r_dx = Com_ID;
 reg.r_ax = com_params;
 intr(0x14, &reg);

 outportb(com_linecontrolport,3);    /* dlab = 0                            */
 outportb(com_modemcontrolport,8);   /* force aux2 control low              */
 outportb(com_interruptenableport,1);/* 8250 interrupts upon data available */
 tailptr  = 0;
 headptr  = 0;
 overflow = 0;

 outportb(0x21, inportb(0x21) & 0xf7); /* 8259 mask - enable int 0bh/ir 3  */
}
/* ------------------------------------------------------------------------ */
void com_wrapup(void){

 outportb(com_interruptenableport, 0); /* 8250 interrupts disabled          */
 outportb(com_modemcontrolport, 0);    /* reset modem control register      */

 setvect(0x0b, oldhandler);            /* restore interrupt 0bh vector       */
 outportb(0x21, inportb(0x21) | 0x08); /* 8259 mask - disable int 0bh - ir 3 */

 outportb(com_linecontrolport, inportb(com_linecontrolport) & 0x7f); /* dlab = 0                           */
 if( (inportb(com_linestatusport) & 1) ) inportb(com_dataport);    /* flush input port                   */
}
/* ------------------------------------------------------------------------ */
void PrintHelp(void){

 printf("\nERROR IN COMMAND LINE OPTIONS\n");

 printf("use: RLoad filename[.hex] [-c=n] [-b=nnnn] [-d=n] [-s=n] [-p=n]\n\n");

 printf("Parameter  Description                 Choices                Default\n");
 printf("---------  -------------------         -----------------      -------\n");
 printf("-c=n       Com Port                    n=1 or n=2                 1\n");
 printf("-b=nnnn    Baud Rate                                           9600\n");
 printf("-d=n       Number of Data Bits         n=7 or n=8;                8\n");
 printf("-s=n       Number of Stop Bits         n=1 or n=2;                1\n");
 printf("-p=n       Parity                      n=0 (  No Parity)           \n");
 printf("                                       n=1 ( Odd Parity)           \n");
 printf("                                       n=2 (Even Parity)          0\n");
 printf("-a         ASCII used for downloading BASIC files                                                    ");
}
/* ------------------------------------------------------------------------ */
int OptionsSetup(int argc, char **argv, char *pcFileName){
unsigned int i;
char         pcFlag[128], pcKey[128];
int          iValue, iError;

 pcFileName[0]=0;
 iError=0;
 for(i=1;i<argc;i++)
  {
   strcpy(pcFlag, argv[i]);
   if( (pcFlag[0]=='-') || (pcFlag[0]=='/') )
    {
     strcpy(pcKey, strtok(pcFlag, "/-"));
     iValue=atoi(pcKey+2);

     switch(tolower(pcKey[0]))
      {
       case 'c' /*Com Port */ : if(iValue>0) Com_ID   = iValue; break;
       case 'b' /*Baud Rate*/ : if(iValue>0) BaudRate = iValue; break;
       case 'd' /*Data Bits*/ : if(iValue>0) DataBits = iValue; break;
       case 's' /*Stop Bits*/ : if(iValue>0) StopBits = iValue; break;
       case 'p' /*Parity   */ : if(iValue>0) Parity   = iValue; break;
       case 'a' /* ASCII   */ : iASCII=1; break;
       default: iError=1; break;
       }
     }
    else
     {
      strcpy(pcFileName, pcFlag);
      }
   }

  if(!iError)
   {
    printf("\nCurrently Selected Options:\n");
    switch(Com_ID)
     {
      case 1 : printf(" Serial Port 1 "); break;
      case 2 : printf(" Serial Port 2 "); break;
      default: printf("Error: Serial Port Must be 1 or 2 - default 1 will be used\n");
      }

    switch(DataBits)
     {
      case 7 : com_params = 2; printf(" 7 Data Bits, "); break;
      case 8 : com_params = 3; printf(" 8 Data Bits, "); break;
      default: com_params = 3;
      	       printf("Error: Data length must be 7 or 8 - default 8 will be used\n");
      }

    switch(StopBits)
     {
      case 1 : printf(" 1 Stop Bit, "); break;
      case 2 : com_params += 4; printf(" 2 Stop Bits, "); break;
      default: printf("Error: Stop bits must be 1 or 2 - default 1 will be used\n");
      }


    switch(Parity)
     {
      case 0 : printf(" No Parity, "); break;
      case 1 : com_params +=  8; printf(" Odd Parity, "); break;
      case 2 : com_params += 24; printf(" Even Parity, "); break;
      default: printf("Error: Parity must be 0 (no parity), 1 (odd parity) or 2 (even parity)\n");
               printf("       default 0 (no parity) will be used\n");
      }

    switch(BaudRate)
     {
      case  110 : printf(" 110 Baud\n");
      case  150 : com_params +=  32; printf(" 110 Baud\n"); break;
      case  300 : com_params +=  64; printf(" 150 Baud\n"); break;
      case  600 : com_params +=  96; printf(" 300 Baud\n "); break;
      case 1200 : com_params += 128; printf(" 600 Baud\n "); break;
      case 2400 : com_params += 160; printf(" 1200 Baud\n"); break;
      case 4800 : com_params += 192; printf(" 4800 Baud\n"); break;
      case 9600 : com_params += 224; printf(" 9600 Baud\n"); break;
      default   : com_params += 224;
		  printf("Error: Baud Rate must be 110, 150, 300, 600, 1200, 2400, 4800, or 9600\n");
		  printf("       default 9600 will be used\n");
      }
    if(iASCII) printf(" Downloading ASCII file\n");
     else printf(" Downloading Intel Hex file\n");
   }
  else PrintHelp();

 com_params &= 0x00ff; /*just to be sure*/

 if(!strlen(pcFileName))
  {
   printf("ERROR - NO FILE SPECIFIED\n");
   iError=1;
   }

 return(iError);
}
/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------ */
int SendASCIIscript(char *sFileName){

char  c;
FILE *fp;

 if(!strchr(sFileName, (int)'.')) strcat(sFileName, ".hex");

 if(!(fp=fopen(sFileName,"rt")))
  {
   printf("ERROR - cannot open file \"%s\"\n",sFileName);
   return(1);
   }
  else printf("Downloading file \"%s\"\n",sFileName);


 while( (c=fgetc(fp))!=EOF)
  {
   while(CharReady()) printf("%c",(char)receive());
   send(c);
   switch(c)
    {
     case 13 : delay(25); break;

     case 10 : delay(25);
               send(13);

               do
                {
                 while(!CharReady());
                 c=receive();
                 printf("%c",c);
                 if(c=='>')break;
                 } while(1);

               delay(50);
               break;

     default : delay(50); break;
     }
   }

 fclose(fp);

 delay(15);
 return(0);
}
/* ------------------------------------------------------------------------ */
int SendScript(char *sFileName){

char  c;
FILE *fp;

 if(!strchr(sFileName, (int)'.')) strcat(sFileName, ".hex");

 if(!(fp=fopen(sFileName,"rt")))
  {
   printf("ERROR - cannot open file \"%s\"\n",sFileName);
   return(1);
   }
  else printf("Downloading file \"%s\"\n",sFileName);


 send('l');
 delay(1);
 send(CR);

 delay(1);
 while( (c=fgetc(fp))!=EOF)
  {
   while(CharReady()) printf("%c",(char)receive());
   if( (c!=CR) && (c!=LF) ) send(c);
   if(c==LF) delay(5); else delay(1);
   }
 fclose(fp);

 delay(5);
 return(0);
}
/* ------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------- */
/*            main                                                           */
/* ------------------------------------------------------------------------- */
int main(int argc, char **argv){

char strz[128];

 printf("\nRLoad Version 1.00\n");
 printf("(C) 1994 Rigel Corporation\n");

 if(OptionsSetup(argc, argv, strz))
  {
   PrintHelp();
   return(-1);
   }

 com_setup();
// setcbrk(0);
 if(!iCheckBoard(10))
  {
   printf("Board NOT Responding... Press RESET and Try Again\n");
   exit(-1);
   }

 if(!iASCII)
  {
   if(SendScript(strz)) PrintHelp();
   }
  else
   {
    if(SendASCIIscript(strz)) PrintHelp();
    }

 if(!iCheckBoard(10))
  {
   printf("Board NOT Responding... Press RESET and Try Again\n");
   exit(-1);
   }
  else printf("Download Completed...\n");
 return(0);
}
/* -------------------------------------------------------------------------- */
